#!/bin/sh
#
# $Id: nightly_dods_build.sh,v 1.3.2.13 2004/06/14 20:57:23 jimg Exp $
#
# Perform a nightly build of the dods code in $DODS_ROOTS. Each
# directory listed in DODS_ROOTS should be the root directory of DODS
# C++ code checked out from CVS. The script will update the code
# getting the latest changes and run make.

# Read the site-specific configuration information. CVS holds an example of
# this file in n ightly_dods_build.conf.example. Rename the file to remove
# `.example' and edit for your site. 
# Ethan changed the script so that it looks for the conf file in 
# $DODS_ROOT/etc; I hacked that change so that it first tests to see if 
# $DODS_ROOT is set and otherwise looks in the CWD. jhrg 04/21/04

if test -n "$DODS_ROOT"
then
    . $DODS_ROOT/etc/nightly_dods_build.conf
else
    . ./nightly_dods_build.conf
fi

host_full=`hostname`
host=`echo $host_full | sed 's/\([^\.]*\)\..*/\1/'`
date=`date +%Y.%m.%d`
platform=`$DODS_ROOT/etc/config.guess`

target=World
make_log=${host}.${platform}.${build}.${target}.${date}

if [ ! -d $DODS_ROOT ]
then
    echo
    echo "Could not cd to $DODS_ROOT" > $make_log
    echo
    exit
fi
cd $DODS_ROOT

echo "Build of ${build} using $build, ${target} on $date" > $make_log
echo "Built on ${host_full}, ${platform} (`uname -a`)" >> $make_log

# If OPTION_N is set, then don't actually do the build, just say what would
# have been done and move the log around. 04/05/04 jhrg

echo "__________ make distclean & grammarclean ______________" >> $make_log
if test -z "$OPTION_N"
then
    $MAKE distclean >> $make_log 2>&1
    (cd src/dap && $MAKE grammarclean >> ../../$make_log 2>&1)
fi

echo "__________ cvs update _________________________________" >> $make_log
if test -z "$OPTION_N" && test "$UPDATE_CVS" = "Yes"
then
    cvs -z3 update -d -P >> $make_log 2>&1
fi

# echo "__________ make source-tar ____________________________" >> $make_log
# $MAKE source-tar >> $make_log 2>&1

echo "__________ configure __________________________________" >> $make_log
if test -z "$OPTION_N"
then
    ./configure >> $make_log 2>&1
fi

echo "__________ make ${target} _____________________________" >> $make_log
if test -z "$OPTION_N"
then
    $MAKE $target >> $make_log 2>&1
fi

echo "__________ make binary-tar_____________________________" >> $make_log
if test -z "$OPTION_N"
then
    $MAKE binary-tar >> $make_log 2>&1
fi

if test -z "$DODS_DONT_RUN_check"
then
    echo "__________ make check _________________________________" >> $make_log
    if test -z "$OPTION_N"
    then
        $MAKE check >> $make_log 2>&1
    fi
fi

echo "_______________________________________________________" >> $make_log
echo "Build completed at $date." >> $make_log 2>&1

# Move the logs around: First move the current log/logs into the 'old' 
# directory. Then delete any log older than ten days. Finally, mv the new log
# into the directory. Then end result is that $new_logs_directory holds
# only the newest log for the machine and the $old_logs_archive holds the
# past ten days logs. 04/05/04 jhrg

mv $new_logs_directory/${host}.${platform}.${build}.${target}* $old_logs_archive

very_old=`find $old_logs_archive -ctime +10`
if test -n "$very_old"
then
    rm -f $very_old
fi

mv $make_log $new_logs_directory

# Create the snapshot tar balls iff $SEPARATE_TARS is set.

if test -n "$SEPARATE_TARS"
then
    # Now move binary files and date stamp them.
    # platform is set at the top of the script now. 10/28/03 jhrg
    bindir=../binary-${build}-${platform}
    mkdir $bindir 2> /dev/null
    rm ${bindir}/*.tgz
    for b in `ls ../*${platform}.tar.gz`
    do
	new_name=`basename $b .tar.gz`.${date}.tgz
	mv $b ${bindir}/${new_name}
    done
fi

# $Log: nightly_dods_build.sh,v $
# Revision 1.3.2.13  2004/06/14 20:57:23  jimg
# Modified so that CVS is updated only when UPDATE_CVS is set to "Yes". This
# should get things going again in the face of the recent CVS break in.
#
# Revision 1.3.2.12  2004/04/21 15:42:49  jimg
# Modified Ethan's change; the script checks $DODS_ROOT and uses it only
# if it's defined. Otherwise it looks in the CWD.
#
# Revision 1.3.2.11  2004/04/14 17:34:48  edavis
# Specify path for nightly_dods_build.conf file.
#
# Revision 1.3.2.10  2004/04/06 21:48:02  jimg
# Changes for the Alpha/OSF 5.1 build. There are also changes in the .conf
# file for this platform.
#
# Revision 1.3.2.9  2004/04/06 15:02:45  jimg
# Added OPTION_N thing/hack. Setting OPTION_N to a non-null string is like
# using -n on make or cvs; it goes through the motions without doing any of the
# actions. For this script, that's not entirely true since it moves the logs
# around, but the option makes it easier to see what's going to happen without
# running the complete build.
# Also, I removed the commented-out code Re: sending email confirmations.
# I also enclosed $DODS_DONT_RUN_check in double quotes for the test.
#
# Revision 1.3.2.8  2004/04/05 22:11:15  edavis
# sparc build script merge
#
# Revision 1.3.2.7  2004/04/05 19:54:09  jimg
# Changes: No longer sends mail and now keeps only the current build log in the
# new_log_directory. THe ten most recent logs are kept in the old_logs_archive
# directory.
#
# Revision 1.3.2.6  2004/02/17 16:15:53  jimg
# Fixed a nasty bug in the script's error detection regular expression.
#
# Revision 1.3.2.5  2003/12/03 18:32:26  edavis
# Add ability to not run the "check" target. If the environment variable
# DODS_DONT_RUN_check is set, the "check" target will not be run.
#
# Revision 1.3.2.4  2003/11/19 17:25:06  jimg
# I fixed up the message sent when an error is found; I think this is
# formatted a bit better.
#
# Revision 1.3.2.3  2003/10/29 18:29:51  jimg
# I fixed the subject line thing in a way that I think will work on machines
# that support mail -s and those that don't. Set $use_mail_subject_option to
# "yes" to use -s; leave it null to use the Subject: line.
#
# Revision 1.3.2.2  2003/10/29 15:34:32  jimg
# I modified how the script looks for errors (using make's error return) so
# that tests like those used for the netCDF code show up when they don't work.
# Also changed the script so that is says everything went OK when that's the
# case. The Subject line thing is not working correctly, but I'll work on that
# later...
#
# Revision 1.3.2.1  2003/06/24 22:10:14  edavis
# Change to mail command that works on chevy and laraine.
#
# Revision 1.3  2003/04/23 23:26:27  jimg
# Merged with 3.3.1.
#
# Revision 1.2.2.1  2003/03/07 22:53:10  jimg
# Modified the script so that it send a more useful error message.
#
# Revision 1.2  2003/01/22 00:12:05  jimg
# Added/Updated from release-3-2 branch.
#
# Revision 1.1.2.8  2002/12/16 22:36:24  jimg
# Added code to move the tar balls into files that contain the platform
# name and date.
#
# Revision 1.1.2.7  2002/09/13 04:38:46  jimg
# Fixed a minor bug where an extra log file was created in the src/dap directory.
#
# Revision 1.1.2.6  2002/04/24 23:29:19  jimg
# I modified the test used to figure out if an error occurred so that code
# which has no Makefile does not get flagged as an error (but it will still
# show up in the logs with three asterisks). The updated configure scripts
# should all be set so that if the program/component cannot be built, no
# Makefile is created.
#
# Revision 1.1.2.5  2002/04/17 23:21:42  jimg
# Added a time/date stamp to the end of the build.
#
# Revision 1.1.2.4  2002/04/12 23:05:53  jimg
# Moved the configuration information to a separate file which this script
# sources. That way simple fixes in this script can easily be used by all the
# builds. The information that's specific to a given site is held in
# nightly_dods_build.conf; there's and example conf file with the additional
# extension '.example'.
#
# Revision 1.1.2.3  2002/04/12 18:17:28  jimg
# Added the DODS_ROOTS variable to the `Set configuration parameters' section
# of the script. If you fail to set this, the script fails mysteriously.
#
# Revision 1.1.2.2  2002/04/12 18:02:48  jimg
# Added variables to make configuration of the log's destination directories
# easier. I also consolidated the configuration information at the front of the
# script (hopefully continuing the work that Ethan did).
#
